@@ -226,24 +226,24 @@ def service(urlpath=None, auto_parse=True, static=False, accel=False,
     return decorator
 
 
-def lookup(urlpath, full=None):
+def lookup(urlpath, full=None, max_recursion=5, max_urlpath_args=5):
 
     path, ext = splitext(urlpath.strip().strip('/'))
     args = []
 
-    while path not in registry:
+    while path not in registry and len(args) <= max_urlpath_args:
         try:
-            path, arg = path.rsplit('/', 1)
-            args.append(arg)
+            path, arg = path.strip().rsplit('/', 1)
+            args.append(arg.strip())
         except ValueError:
             break
 
     fun = registry.get(path, None)
 
-    if not fun:
+    if not fun and max_recursion > 0:
         try:
             prefix, urlpath = urlpath.split('/', 1)
-            return lookup(urlpath, full or urlpath)
+            return lookup(urlpath, full or urlpath, max_recursion - 1)
         except ValueError:
             fun = registry.get('', None)
             args = [full]
@@ -257,11 +257,16 @@ def lookup(urlpath, full=None):
     return fun, args, ext, out
 
 
-def consume(_state_, urlpath, *args, **argd):
+def consume(_state_, service_call, *args, **argd):
     start = timestamp()
     try:
         fun = error = exception = None
-        fun, argv, ext, out = lookup(urlpath)
+
+        if isinstance(service_call, basestring):
+            urlpath = service_call
+            fun, argv, ext, out = lookup(normpath(service_call))
+        else:
+            urlpath, fun, argv, ext, out = service_call
 
         if not callable(fun):
             raise exc.NotFound(urlpath)
