@@ -26,6 +26,7 @@ SERVER = 'Fanery/%s' % __version__
 CONTENT_TYPE = 'text/plain'
 CHARSET = 'utf8'
 ENCODING_ERROR = 'Wrong request encoding, must be %s.' % CHARSET
+X_CALL_HEADER = 'X-Fanery-Call'
 STRICT_HTML_HEADERS = {
     'X-Frame-Options': 'SAMEORIGIN',
     'X-XSS-Protection': '1; mode=block',
@@ -112,10 +113,11 @@ def handler(req):
             argd.update(parse_json(params['_json_']))
 
         # build request state
+        req_headers = req.headers
         _state_ = State(domain=req.host.split(':', 1)[0],
-                        origin=req.client_addr or '127.0.0.1',
-                        sid=None, profile=None, role=None,
-                        ssl=bool(req.headers.get('X-Forwarded-Proto', req.scheme) == 'https'))                  # noqa
+                        origin=req_headers.get('X-Real-IP', None) or req.client_addr or '127.0.0.1',            # noqa
+                        ssl=bool(req_headers.get('X-Forwarded-Proto', req.scheme) == 'https'),                  # noqa
+                        sid=None, profile=None, role=None)
 
         # trigger service reload
         if is_development:
@@ -125,8 +127,8 @@ def handler(req):
         fun, argv, ext, out = lookup(normpath(path))
 
         # RFD and XSS attack mitigation against JSON/JSONP Web APIs
-        if fun.safe and not is_development:
-            assert req.headers.get('X-Fanery-Call', None) == 'true', 'api-call'
+        if (fun and (fun.safe or not fun.static)) and not is_development:
+            assert req_headers.get(X_CALL_HEADER, None) == 'true', 'api-call'
             headers['Content-Disposition'] = 'attachment; filename=f.txt'
 
         # perform service call
